@@ -233,7 +233,7 @@ static void edit_params(u32 argc, char** argv) {
       FATAL("ASAN and MSAN are mutually exclusive");
 
     if (getenv("AFL_HARDEN"))
-      FATAL("ABSAN and AFL_HARDEN are mutually exclusive");
+      FATAL("ASAN and AFL_HARDEN are mutually exclusive");
 
     cc_params[cc_par_cnt++] = "-U_FORTIFY_SOURCE";
     cc_params[cc_par_cnt++] = "-fsanitize=address";
@@ -323,7 +323,6 @@ int main(int argc, char** argv) {
 
   }
 
-
   find_as(argv[0]);
 
   edit_params(argc, argv);
